Package: scECODA
Title: Single-Cell Exploratory Compositional Data Analysis
Version: 1.1.5
Authors@R: c(
    person("Christian", "Halter",
      email = "scecoda.dev@gmail.com",
      role = c("aut", "cre"),
      comment = c(ORCID = '0009-0009-5479-2246')),
    person("Massimo", "Andreatta",
      email = "Massimo.Andreatta@unige.ch",
      role = c("aut"),
      comment = c(ORCID = '0000-0002-8036-2647')),
    person("Santiago", "Carmona",
      email = "Santiago.Carmona@unige.ch",
      role = c("aut"),
      comment = c(ORCID = '0000-0002-2495-0671')),
    person("Swiss Cancer Research Foundation", role = c("fnd"))
    )
Description: The scECODA R package provides a complete workflow for the 
    analysis and visualization of compositional data, primarily focusing on 
    cell type proportions derived from single-cell data. It implements 
    specialized methods, such as the Centered Log-Ratio (CLR) transformation, 
    to properly analyze proportional data while avoiding the biases introduced 
    by the compositional constraint. The package encapsulates data management, 
    transformation, and analysis into a single SummarizedExperiment object,
    offering downstream tools for dimensionality reduction via PCA, calculating 
    critical metrics like the Adjusted Rand Index (ARI) and Modularity to 
    quantify sample grouping quality, and generating high-quality visualizations
    like heatmaps and scatter plots.
biocViews: Software, SingleCell, Transcriptomics, CellBasedAssays,
        Normalization, Preprocessing, Visualization, Clustering,
        DimensionReduction, FeatureExtraction, PrincipalComponent
BugReports: https://github.com/carmonalab/scECODA/issues
License: GPL-3 + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Depends: R (>= 4.6.0)
VignetteBuilder: knitr
Suggests: Seurat (>= 5.0.0), igraph, knitr, rmarkdown, BiocStyle,
        testthat, scRNAseq
Config/testthat/edition: 3
URL: https://github.com/carmonalab/scECODA
Imports: BiocGenerics, cluster, corrplot, DESeq2, dplyr, factoextra (>=
        2.0.0), ggplot2, ggpubr, ggrepel, gtools, Matrix, mclust,
        methods, pheatmap, plotly, rlang, rstatix, S4Vectors, stringr,
        SummarizedExperiment (>= 1.34.0), tidyr, vegan
git_url: https://git.bioconductor.org/packages/scECODA
git_branch: devel
git_last_commit: 4af79f0
git_last_commit_date: 2026-05-13
Repository: Bioconductor 3.24
Date/Publication: 2026-05-13
NeedsCompilation: no
Packaged: 2026-05-14 00:03:09 UTC; biocbuild
Author: Christian Halter [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-5479-2246>),
  Massimo Andreatta [aut] (ORCID:
    <https://orcid.org/0000-0002-8036-2647>),
  Santiago Carmona [aut] (ORCID: <https://orcid.org/0000-0002-2495-0671>),
  Swiss Cancer Research Foundation [fnd]
Maintainer: Christian Halter <scecoda.dev@gmail.com>
