# Generated by roxygen2: do not edit by hand

export(calc_anosim)
export(calc_ari)
export(calc_clr)
export(calc_freq)
export(calc_modularity)
export(calc_sil)
export(calculate_pseudobulk)
export(deseq2_normalize)
export(ecoda)
export(find_hvcs)
export(get_celltype_counts)
export(get_celltype_variances)
export(get_hvcs)
export(get_sample_metadata)
export(plot_barplot)
export(plot_boxplot)
export(plot_corr)
export(plot_heatmap)
export(plot_pca)
export(plot_pca3d)
export(plot_varmean)
export(replace_zeros)
importFrom(BiocGenerics,counts)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,estimateSizeFactors)
importFrom(DESeq2,vst)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,tcrossprod)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(corrplot,corrplot)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(factoextra,fviz_pca)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitterdodge)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggpubr,ggboxplot)
importFrom(ggpubr,stat_compare_means)
importFrom(ggpubr,stat_pvalue_manual)
importFrom(ggrepel,geom_text_repel)
importFrom(gtools,mixedsort)
importFrom(mclust,adjustedRandIndex)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(pheatmap,pheatmap)
importFrom(plotly,add_markers)
importFrom(plotly,add_paths)
importFrom(plotly,group2NA)
importFrom(plotly,plot_ly)
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(rstatix,add_xy_position)
importFrom(rstatix,wilcox_test)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,hclust)
importFrom(stats,prcomp)
importFrom(stats,reformulate)
importFrom(stats,var)
importFrom(stringr,str_to_title)
importFrom(tidyr,everything)
importFrom(tidyr,pivot_longer)
importFrom(vegan,anosim)
